package cd.casic.ci.common.pipeline.container;

import cd.casic.ci.common.pipeline.IModelTemplate;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.time.BuildRecordTimeCost;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;
import lombok.experimental.SuperBuilder;

import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.container
 * @Project：ops-pro
 * @name：Container
 * @Date：2025/03/25 16:53
 * @Filename：Container
 * @description：Todo
 */
@Data
@SuperBuilder
@Schema(title = "流水线模型-多态基类")
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "@type")
@JsonSubTypes({
        @JsonSubTypes.Type(value = TriggerContainer.class, name = TriggerContainer.classType),
        @JsonSubTypes.Type(value = NormalContainer.class, name = NormalContainer.classType),
        @JsonSubTypes.Type(value = VMBuildContainer.class, name = VMBuildContainer.classType)
})
@Getter
@AllArgsConstructor
public abstract class Container implements IModelTemplate {
    private String id;
    private String name;
    private List<Element> elements;
    private String status;
    @Deprecated
    private Long startEpoch;
    @Deprecated
    private Long systemElapsed;
    @Deprecated
    private Long elementElapsed;
    private Boolean canRetry;
    private String containerId;
    private String containerHashId;
    private String startVMStatus;
    private Integer executeCount;
    private String jobId;
    private Boolean containPostTaskFlag;
    private Boolean matrixGroupFlag;
    private BuildRecordTimeCost timeCost;
    private Integer startVMTaskSeq;

    public void resetBuildOption(int executeCount) {
        this.status = null;
        this.timeCost = null;
        this.startEpoch = null;
        this.elementElapsed = null;
        this.systemElapsed = null;
        this.startVMStatus = null;
        this.executeCount = executeCount;
    }

    public void transformCompatibility() {
        if (elements != null) {
            elements.forEach(Element::transformCompatibility);
        }
    }

    public Map<String, Object> genTaskParams() {
        Map<String, Object> configParams = JSONUtil.parseObj(this).toBean(Map.class);
        if (elements != null && !elements.isEmpty()) {
            configParams.put("elements", List.of());
        }
        return configParams;
    }

    public abstract String getClassType();

    public abstract Container getContainerById(String vmSeqId);

    public abstract void retryFreshMatrixOption();

    public abstract List<Container> fetchGroupContainers();

    public abstract Map<String, String> fetchMatrixContext();

    public abstract boolean containerEnabled();

    public abstract void setContainerEnable(boolean enable);
}