package cd.casic.ci.common.pipeline.container;

import cd.casic.ci.common.pipeline.NameAndValue;
import cd.casic.ci.common.pipeline.option.JobControlOption;
import cd.casic.ci.common.pipeline.option.MatrixControlOption;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.time.BuildRecordTimeCost;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.container
 * @Project：ops-pro
 * @name：NormalContainer
 * @Date：2025/03/25 16:54
 * @Filename：NormalContainer
 * @description：Todo
 */
@Data
@NoArgsConstructor
@Schema(title = "流水线模型-普通任务容器")
public class NormalContainer implements Container {
    public static final String classType = "normal";

    @Schema(title = "构建容器序号id", required = false, readOnly = true)
    private String id;
    @Schema(title = "容器名称", required = true)
    private String name;
    @Schema(title = "任务集合", required = true)
    private List<Element> elements;
    @Schema(title = "容器状态", required = false, readOnly = true)
    private String status;
    @Schema(title = "系统运行时间", required = false, readOnly = true)
    @Deprecated
    private Long startEpoch;
    @Schema(title = "系统耗时（开机时间）", required = false, readOnly = true)
    @Deprecated
    private Long systemElapsed;
    @Schema(title = "插件执行耗时", required = false, readOnly = true)
    @Deprecated
    private Long elementElapsed;
    @Schema(title = "允许可跳过", required = false)
    @Deprecated
    private Boolean enableSkip;
    @Schema(title = "触发条件", required = false)
    @Deprecated
    private List<NameAndValue> conditions;
    @Schema(title = "是否可重试-仅限于构建详情展示重试，目前未作为编排的选项，暂设置为null不存储", required = false, readOnly = true)
    private Boolean canRetry;
    @Schema(title = "构建容器顺序ID（同id值）", required = false, readOnly = true)
    private String containerId;
    @Schema(title = "容器唯一ID", required = false, readOnly = true)
    private String containerHashId;
    @Schema(title = "无构建环境-等待运行环境启动的排队最长时间(分钟)", required = false)
    @Deprecated
    private Integer maxQueueMinutes = 60;
    @Schema(title = "无构建环境-运行最长时间(分钟)", required = false)
    @Deprecated
    private Integer maxRunningMinutes = 1440;
    @Schema(title = "流程控制选项", required = true)
    private JobControlOption jobControlOption;
    @Schema(title = "互斥组", required = false)
    private MutexGroup mutexGroup;
    @Schema(title = "构建环境启动状态", required = false, readOnly = true)
    private String startVMStatus;
    @Schema(title = "容器运行次数", required = false, readOnly = true)
    private Integer executeCount;
    @Schema(title = "用户自定义ID", required = false, hidden = false)
    private String jobId;
    @Schema(title = "是否包含post任务标识", required = false, readOnly = true)
    private Boolean containPostTaskFlag;
    @Schema(title = "是否为构建矩阵", required = false, readOnly = true)
    private Boolean matrixGroupFlag;
    @Schema(title = "各项耗时", required = true)
    private BuildRecordTimeCost timeCost;
    @Schema(title = "开机任务序号", required = false, readOnly = true)
    private Integer startVMTaskSeq;
    @Schema(title = "构建矩阵配置项", required = false)
    private MatrixControlOption matrixControlOption;
    @Schema(title = "所在构建矩阵组的containerHashId（分裂后的子容器特有字段）", required = false)
    private String matrixGroupId;
    @Schema(title = "当前矩阵子容器的上下文组合（分裂后的子容器特有字段）", required = false)
    private Map<String, String> matrixContext;
    @Schema(title = "分裂后的容器集合（分裂后的父容器特有字段）", required = false)
    private List<NormalContainer> groupContainers;

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public Container getContainerById(String vmSeqId) {
        if (id.equals(vmSeqId) || containerId.equals(vmSeqId)) return this;
        if (groupContainers != null) {
            for (Container container : groupContainers) {
                if (container.getId().equals(vmSeqId) || container.getContainerId().equals(vmSeqId)) return container;
            }
        }
        return null;
    }

    @Override
    public void retryFreshMatrixOption() {
        groupContainers = new ArrayList<>();
        if (matrixControlOption != null) {
            matrixControlOption.setFinishCount(null);
            matrixControlOption.setTotalCount(null);
        }
    }

    @Override
    public List<Container> fetchGroupContainers() {
        return groupContainers != null ? new ArrayList<>(groupContainers) : null;
    }

    @Override
    public Map<String, String> fetchMatrixContext() {
        return matrixContext;
    }

    @Override
    public Boolean containerEnabled() {
        return jobControlOption != null ? jobControlOption.getEnable() : true;
    }

    @Override
    public void setContainerEnable(Boolean enable) {
        jobControlOption = jobControlOption != null ? jobControlOption.copy(enable) : new JobControlOption(enable);
    }

    @Override
    public void resetBuildOption(int executeCount) {

    }

    @Override
    public void transformCompatibility() {
        if (jobControlOption != null && jobControlOption.getTimeoutVar() == null || jobControlOption.getTimeoutVar().isBlank()) {
            jobControlOption.setTimeoutVar(String.valueOf(jobControlOption.getTimeout()));
        }
        if (mutexGroup != null && (mutexGroup.getTimeoutVar() == null || mutexGroup.getTimeoutVar().isBlank())) {
            mutexGroup.setTimeoutVar(String.valueOf(mutexGroup.getTimeout()));
        }
    }

    @Override
    public Map<String, Object> genTaskParams() {
        return null;
    }

    @Override
    public String getTemplate() {
        return null;
    }

    @Override
    public String getRef() {
        return null;
    }

    @Override
    public Map<String, String> getVariables() {
        return null;
    }
}
